# -*- coding: utf8 -*-

# BMCLAPI文档地址: https://bmclapidoc.bangbang93.com

class BMCLAPI:
    mojang_bmclapi_domain_map = {
        'launchermeta.mojang.com': 'bmclapi2.bangbang93.com',
        'launcher.mojang.com': 'bmclapi2.bangbang93.com',
        'resources.download.minecraft.net': 'bmclapi2.bangbang93.com/assets',
        'libraries.minecraft.net': 'bmclapi2.bangbang93.com/maven',
    }